"""Metrics and visualizations for evaluating chatbot's AI utilization.

This project intents to provide techniques for evaluating the quality of a chatbot's AI knowledge base
and visualize insights from chatbot message bases.
"""

__author__ = 'squad ROps'
__email__ = 'anaytics.dar@take.net'
__version__ = '0.2.0'
__license__ = 'MIT License'
__credits__ = ['Cecilia Assis', 'Gabriel Salgado', 'Juliana Guamá', 'Milo Utsch', 'Rogers Damas']
__all__ = ['AiEvaluation']

from .domain import AiEvaluation
